unit TyperPanel;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, stdctrls, Global, EndDialog, inifiles;

type
//TFileNameString = type String;
  TTyperPanel = class(TWinControl)
  private
    ReadEdit : TEdit;
    WriteEdit : TLabel;
    FFileName : {TFileNameString} String;
    Row : String;
    RowLength : integer;
    RowRead : String;
    TypeIndex : integer;
    Book : TextFile;
    FMaxChars : integer;
    IlFIleIni : TIniFile;
    function GetLine : String;
    procedure WMSize(var Message: TWMSize);  message WM_SIZE;
    procedure Repaint;
    procedure SetFont(S : String);
    function GetFont : String;
    procedure ReadIni;
    procedure SaveIni;

  procedure OpenFile(FiletoRead : String);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Open;
    procedure KeyPressed(Sender: TObject; var Key: Char);
    function GetCurrentChar : String;
  published
    property FileName : {TFileNameString} String read FFileName write FFileName;
    property MaxChars : integer read FMaxChars write FMaxChars;
    property FontName : String read GetFont write SetFont;
  end;


var CharSet : Set of Char = [
             'q', 'w','e', 'r', 't', 'y', 'u', 'i', 'o', 'p',
             'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',
             'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '-', ' '];
procedure Register;

implementation

constructor TTyperPanel.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Caption := '';
  ReadEdit := TEdit.Create(self);
  WriteEdit := TLabel.Create(self);
  ReadEdit.Enabled := False;
  ReadEdit.Visible := True;
  WriteEdit.Font.Color := clBlack;
  WriteEdit.Visible := True;
  ReadEdit.Parent := self;
  WriteEdit.Parent := self;
  Repaint;
  TypeIndex := 1;
  RowRead := '';
end;

function TTyperPanel.GetLine : String;

  procedure StripSpaces(var ToStrip : String);
  var prevspace : Boolean;
      i : integer;
  begin
    prevspace := False;
    for i := 1 to Length(ToStrip) do
      if ToStrip[i] = ' ' then
        if prevSpace then
          Delete(ToStrip,i,1)
        else
          prevspace := true
      else
        prevspace := false
  end;  { StripSpaces }

  procedure CutIllegalCharacters(var ToCut : String);
  var i : integer;
  begin
    ToCut := LowerCase(ToCut);
    for i := 1 to Length(ToCut) do
      if  not (ToCut[i] in CharSet) then
        ToCut[i] := ' ';
  end;

begin
  if eof(Book) then
    OKBottomDlg.Show
  else begin
    repeat
      Readln(Book,Row);
      CutIllegalCharacters(Row);
      StripSpaces(Row);
    until Row <> '';
    Row := LowerCase(Row);
    RowLength := Length(Row);
    ReadEdit.Text := Row;
    result := Row;
  end;
end;

procedure TTyperPanel.OpenFile(FiletoRead : String);
var
  OpenDlg: TOpenDialog;
begin
  AssignFile(Book,FileToRead);
  try
    Reset(Book);
    GetLine;
  except on EInOutError do begin
    MessageDlg('Cannot open file '+FiletoRead, mtError, [mbOk], 0);
    OpenDlg := TOpenDialog.Create(self);
    OpenDlg.Filter := 'Text files (*.txt)|*.TXT';
    if (OpenDlg.Execute) then begin
      FileName := OpenDlg.FileName;
      OpenFile(FileName)
    end else Halt;
  end; {begin}
  end; {except}
end; { TTyperPanel.OpenFile }

procedure TTyperPanel.Repaint; // Could probably have been done more simply
var SideMargin, TopMargin, EditHeight,Length : integer;
begin
  SideMargin := Round(Width/20);
  TopMargin := Round(Height/8);
  EditHeight := Round(Height/8);
  if EditHeight >= 24 then EditHeight := 24;
  Length := Round(Width/10*9);
  ReadEdit.Left := SideMargin;
  WriteEdit.Left := SideMargin;
  WriteEdit.Top := TopMargin;
  ReadEdit.Top := TopMargin*3+EditHeight*2;
  WriteEdit.Font.Size := EditHeight;
  WriteEdit.Font.Pitch := fpFixed;
  ReadEdit.Font.Size := EditHeight;
  ReadEdit.Font.Pitch := fpFixed;
  ReadEdit.Width := Length;
  WriteEdit.Width := Length;
end;  { TTyperPanel.Repaint }

procedure TTyperPanel.WMSize(var Message: TWMSize);
begin
  Repaint;
end;

procedure TTyperPanel.Open;
begin
  ReadIni;
  OpenFile(FileName);
end;

procedure TTyperPanel.KeyPressed(Sender: TObject; var Key: Char);
begin
  if (FirstCharacter) then begin
    FirstCharacter := False;
    StartTime := DateTimeToTimeStamp(Now);
  end;
  if Key = Row[TypeIndex] then begin
    Inc(Characters);
    RowRead := RowRead + Key;
    if TypeIndex = RowLength then begin
      RowRead := '';
      GetLine;
      TypeIndex := 1;
    end else if TypeIndex = FMaxChars then begin
      Row := Copy(Row,TypeIndex+1,RowLength-TypeIndex);
      ReadEdit.Text := Row;
      RowRead := '';
      RowLength := RowLength - TypeIndex;
      TypeIndex := 1;
    end
    else begin
      ReadEdit.Selstart := TypeIndex-1;
      ReadEdit.SelLength := 1;
      ReadEdit.SelText := '*';
      Inc(TypeIndex);
    end;
  end else
    Inc(Mistakes);
    WriteEdit.Caption := RowRead;
end;

procedure TTyperPanel.SetFont(S : String);
begin
  ReadEdit.Font.Name := S;
  WriteEdit.Font.Name := S;
end;

function TTyperPanel.GetFont : String;
begin
  result := WriteEdit.Font.Name;
end;

function TTyperPanel.GetCurrentChar : String;
var S :String;
begin
  SetLength(S,1);
  S[1]:=Row[TypeIndex];
  if S = ' ' then
    result := 'SPACE'
  else
    result := UpperCase(S);
end;

procedure TTyperPanel.SaveIni;
begin
  IlFileIni.WriteString('FileName','Filename',FileName);
  IlFileIni.Free;
end;

procedure TTyperPanel.ReadIni;
var IniFile :String;
    WindowString : String;
begin
  SetLength(WindowString,MAX_PATH+1);
  SetLength(WindowString,GetWindowsDirectory(PChar(WindowString),MAX_PATH));
  IniFile := WindowString+'\typpan.ini';
  ILFileIni := TIniFile.Create(IniFile);
  FileName := IlFileIni.ReadString('FileName','Filename','readme.txt');
  if Pos('\',Filename) = 0 then
    FileName := GetCurrentDir + '\'+FileName;
end;

destructor TTyperPanel.Destroy;
begin
  SaveIni;
  WriteEdit.Free;
  ReadEdit.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTyperPanel]);
{  RegisterPropertyEditor(TypeInfo(TFileNameString), TTyperPanel, 'FileName',
                                                    TFileNameStringProperty);}
end;

end.
